data "aws_sns_topic" "cloudmon" {
  name = var.sns_topic_name
}

resource "aws_cloudwatch_metric_alarm" "lambda_throttles_alarm" {
  for_each = local.lambdas

  alarm_name          = "${local.prefix}-${each.key}-throttle-alarm"
  alarm_description   = "Alert staff when throttles occurs on cloud-monitor project's lambdas."
  comparison_operator = "GreaterThanOrEqualToThreshold"
  evaluation_periods  = 1
  period              = 300
  threshold           = 1
  namespace           = "AWS/Lambda"
  statistic           = "Sum"
  alarm_actions       = [data.aws_sns_topic.cloudmon.arn]
  metric_name         = "Throttles"

  dimensions = {
    FunctionName = aws_lambda_function.crud[each.key].function_name
  }
  tags = local.common_tags
}


resource "aws_cloudwatch_metric_alarm" "lambda_error_alarm" {
  for_each = local.lambdas

  alarm_name          = "${local.prefix}-${each.key}-error-alarm"
  alarm_description   = "Alert staff when errors occurs on cloud-monitor project's lambdas."
  comparison_operator = "GreaterThanOrEqualToThreshold"
  evaluation_periods  = 1
  period              = 300
  threshold           = 1
  namespace           = "AWS/Lambda"
  statistic           = "Sum"
  alarm_actions       = [data.aws_sns_topic.cloudmon.arn]
  metric_name         = "Errors"

  dimensions = {
    FunctionName = aws_lambda_function.crud[each.key].function_name
  }
  tags = local.common_tags
}

resource "aws_lambda_function" "slack" {
  filename                       = data.archive_file.lambda_slack_file.output_path
  function_name                  = "${local.prefix}-slack-messaging"
  role                           = aws_iam_role.iam_for_lambda.arn
  handler                        = "slack.lambda_handler"
  timeout                        = 10
  source_code_hash               = data.archive_file.lambda_slack_file.output_base64sha256
  runtime                        = "python3.8"
  reserved_concurrent_executions = 2
  layers = [
    "arn:aws:lambda:eu-west-1:580247275435:layer:LambdaInsightsExtension:14"
  ]

  environment {
    variables = {
      SLACK_WEBHOOK_URL = var.slack_webhook_url
    }
  }

  tags = local.common_tags
}

data "archive_file" "lambda_slack_file" {
  type        = "zip"
  output_path = "${local.lambda_loc}/zip/slack.zip"
  source_file = "${local.lambda_loc}/slack/slack.py"
}

resource "aws_sns_topic_subscription" "topic_lambda" {
  topic_arn = data.aws_sns_topic.cloudmon.arn
  protocol  = "lambda"
  endpoint  = aws_lambda_function.slack.arn
}

resource "aws_lambda_permission" "with_sns" {
  statement_id  = "AllowExecutionFromSNS"
  action        = "lambda:InvokeFunction"
  function_name = aws_lambda_function.slack.arn
  principal     = "sns.amazonaws.com"
  source_arn    = data.aws_sns_topic.cloudmon.arn
}
